use axum::{
    http::header::SET_COOKIE,
    response::{AppendHeaders, IntoResponse, Redirect},
    routing::post,
    Form, Router,
};

use lettre::message::Mailbox;
use lettre::transport::smtp::AsyncSmtpTransport;
use lettre::{AsyncTransport, Message, Tokio1Executor};
use serde::Deserialize;
use std::error::Error;
use std::net::SocketAddr;

#[derive(Deserialize)]
struct FormData {
    name: String,
    email: String,
    message: String,
}

#[tokio::main]
async fn main() {
    let app = Router::new().route("/submit-email", post(handle_form));

    let addr = SocketAddr::from(([0, 0, 0, 0], 4200));
    println!("Listening on {}", addr);

    let listener = tokio::net::TcpListener::bind("0.0.0.0:4200").await.unwrap();
    axum::serve(listener, app).await.unwrap();
}

async fn handle_form(Form(form_data): Form<FormData>) -> impl IntoResponse {
    match send_email(form_data).await {
        Ok(_) => (
            AppendHeaders([(SET_COOKIE, "is=ok")]),
            Redirect::to("/contact.html"),
        ),
        Err(_) => (
            AppendHeaders([(SET_COOKIE, "is=err")]),
            Redirect::to("/contact.html"),
        ),
    }
}

async fn send_email(form_data: FormData) -> Result<(), Box<dyn Error>> {
    let email = Message::builder()
        .from("webmail@distrust.co".parse()?)
        .reply_to(form_data.email.parse::<Mailbox>()?)
        .to("sales@distrust.co".parse()?)
        .subject("New Website Form Inquiry")
        .body(format!(
            "Name: {}/nEmail: {}/nMessage: {}",
            form_data.name, form_data.email, form_data.message
        ))?;

    let credentials = lettre::transport::smtp::authentication::Credentials::new(
        std::env::var("WEBMAIL_USERNAME")?,
        std::env::var("WEBMAIL_PASSWORD")?,
    );

    let mailer = AsyncSmtpTransport::<Tokio1Executor>::relay("smtp.distrust.co")?
        .credentials(credentials)
        .build();

    mailer.send(email).await?;

    Ok(())
}
